function new=applyMycRules(str, dt, s)
% applyMycRules: applies the production rules of an l-system string for mycorrhizal segments.
%
% Parameters:
% str                   l-system string
% dt                    time step
%
% new                   l-system string
%
% See also: letter
%
% Copyright 2008-2013 Daniel Leitner. See license.txt for details.
%

new=[];
rind = find(str(:,1)>=300 & str(:,1)~=305); % indices of dynamic letters

if isempty(rind) %
    new = str;
    return
end

j=1;
i0 = 1;

while j<=length(rind)
    
    i1 = rind(j);
    
    new = [new; str(i0:i1-1,:)];  % static letters
    
    if str(i1,1)<400 % root system letter
        new = [new; applyRules(str(i1:i1+str(i1,2)-1,:),dt);]; % dynamic letters
    else % myc letter
        new = [new; applyMycRule(str(i1:i1+str(i1,2)-1,:),dt,s);]; % dynamic letters
    end
    i0=i1+str(i1,2);
    
    while j<=length(rind) && i0>rind(j) % letters within letters are skipped
        j=j+1;
    end
    
end

new = [new; str(i0:end,:)];  % final static letters



function nl = applyMycRule(l, dt,s)
% applyRule: applies production rule of a single l-system letter
%
% l         l-system letter
% dt        time step
%
% nl        l-system string
%
% Copyright 2008-2014 Daniel Leitner. See license.txt for details.
%

global mycParameters; % parameter set for mycorrhiza

id = l(1);

switch id
    
    case 400 % MYC SECTION GROWTH
        
        % Parameters
        len = l(1,3); % length
        t = l(1,4); % age of section
        type = l(1,5); % myc type
        iid = l(1,6); % hyphal brach identifier
        pos = getPosition(l(2:4,:));
        pgf = l(5,1:3); % parmaters for the growth function
        ptr = l(5,4:6); % parameters for tropism
        t0 = l(6,1); % section start time
        tend = l(6,2); % section end time
        l0 = l(6,3); % section start time
        lend = l(6,4); % section end times
        rlt = l(6,5); % root life time
        dx = l(6,6); % axial resolution
        heading = l(7,1:3); % initial heading of the root
        b_age = l(7,4);   % 1/branching rate constant
        theta = l(7,5); % branching angle for dichotomous branching
        thetas = l(7,6); % standard deviation of branching angle for dichotomous branching
        
        ls = lend-l0; % section length
        A = min(l0+len+dx,lend) <= growthFunction(t0+t+dt,pgf);
        
        if t0+t < rlt
            
            if t0+t+dt <= b_age
                
                if  A && (len+dx < ls) % new segments
                    
                    l(1,3) = len+dx;
                    lf = letter('F',[dx,pos.t,type+100,iid]);
                    lt = tropism(ptr,heading,pos,dx);
                    l(2:4,:) = updatePos2(dx,lt,l(2:4,:)); % special fast update
                    nl = [lt;lf;applyMycRule(l, dt)];
                    
                elseif A && (len+dx >= ls) % there is only apical zone, no next segment but dichotomous branching
                    
                    nl = letter('F',[0,pos.t+dt,type+100,iid]);
                    
                    %                     lr = ls-len;
                    %                     ct = t0 + t + dt - tend; % Initialize 'next' with ct
                    %                     if (lr>0)
                    %                         next = l(8:l(1,2),:); % copy successor
                    %                         pos.t = pos.t+dt-ct;
                    %                         lf = letter('F',[lr,pos.t,type+100,iid]);
                    %                         lt = tropism(ptr,heading,pos,s);
                    %                         nl = [lt;lf;next];
                    %                         nl = updatePos(nl,pos); % update position
                    %                         nl = applyMycRules(nl,ct,s);
                    %                     else
                    %                         next = l(8:l(1,2),:); % copy successor
                    %                         pos.t = pos.t+dt-ct;
                    %                         next = updatePos(next,pos);
                    %                         nl = applyMycRules(next,ct,s);
                    %                     end
                    
                else % do nothing (increase time)
                    
                    l(1,4) = t+dt; % section age
                    l(4,2) = pos.t+dt; % root age
                    nl = l;
                    
                end
                
            else   % dichotome branching
                
                ttb = b_age - (t0+t); % time til branching
                tab = (t0+t+dt)-b_age ; % time left after branching
                
                %fprintf('%g ttb, %g tab\n',ttb, tab);
                
                % new segments
                if ttb>0
                    nl = applyMycRules(l,ttb,s);
                    hi = find(nl(:,1)==400);
                    l  = nl(hi:end,:);
                    nl = nl(1:hi-1,:); % delete l
                end
                
%                 rt = b_age-(l(6,1)+ l(1,4)); % - (t0+t)                
%                 lr = pgf(1)*rt; % assuming consant growth !!                
%                 lf = letter('F',[lr,pos.t,type+100,iid]);
%                 lt = tropism(ptr,heading,pos,dx);
%                 nl = [nl;lt;lf];                             
                                
                % branching
                myc = letter(403,[type,rlt-(t0+t+ttb),iid],[],pos); 
                
                nl = [nl; letter('['); letter('+',theta+randn(s)*thetas); myc; ...
                    letter(']'); letter('['); letter('-',theta+randn(s)*thetas); ...
                    myc; letter(']')];
                
                nl = updatePos(nl,pos);
                nl = applyMycRules(nl, tab, s);
                
                
            end
            
        else   % branch is dead and stops growing
            
            nl = letter('F',[0,pos.t+dt,type+100,iid]);
            
        end
        
    case 403 % RULE CREATE MYC
        
        % Parameters
        type = l(1,3);
        iid = l(1,5);
        p = mycParameters{l(1,3)};
        
        % apply standard deviation to parameters
        pos = getPosition(l(2:4,:));
        la = max(p.la(1)+randn(s)*p.la(2),0); % length of apical zone
        lb = max(p.lb(1)+randn(s)*p.lb(2),0); % length of basal zone
        nob = max(p.nob(1)+round(randn(s)*p.nob(2)),0); % number of branches
        ln = max(p.ln(1)+randn(s,nob-1,1)*p.ln(2),0); % inter-branch distance
        a = max(p.a(1)+randn(s)*p.a(2),0); % root radius
        theta = max(p.theta(1)+randn(s)*p.theta(2),0); % inter-root angle
        r = max(p.r(1)+randn(s)*p.r(2),0); % initial growth speed
        rlt=max(p.rlt(1)+randn(s)*p.rlt(2),0);  %root life time
        if l(1,4)~=0    % 1.Zeile, 4. Spalte: dort rlt-Alter von Hyphae
            rlt = l(1,4);
        end
        dx = p.dx(1); % resolution along root axis
        k = la+lb+sum(ln); % total length of fully grown root
        pgf = [p.gf(1), r, k]; % parameters for the growth function
        ptr = p.tropism; % parameters for tropism
        b_age = max(1./(p.b(1) + rand(s).*p.b(2)),0); % age of next dichotome branching
        
        if rlt>0
            
            % if the root has a life span
            die = [];
            if (rlt~=inf)
                dc = letter('C',[0,0,0]);  % [0,0,0]   % to do if ent-infizierung: 6the column will be set to tell whether neets to be de-infected, then search from outside for this segment and then the root segment before is the one
                die = letter(301,[0,rlt],dc,pos); % delay
            end
            
            % initial string
            initial = [letter('['); letter('#',a); letter('C',p.color); ...
                die; letter('\',rand(s)*2*pi); letter('+',theta)];
            [~,pos] = updatePos(initial,pos);
            
            % create hyphae
            if r>0 && k>0
                
                if la>0 % apical zone
                    l0 = k-la;
                    lend = k;
                    t0 = growthFunction(l0,pgf,pos,true);
                    tend = growthFunction(lend,pgf,pos,true);
                    apical = letter(400,[0,0,type,iid],[pgf,ptr;t0,tend,l0,lend,rlt,dx;...
                        pos.R(:,1)' b_age p.theta_dicho(1) p.theta_dicho(2); letter(305)] ); % section growth
                else
                    apical = [];
                end
                
                if false %nob>0 % branching zone
                    %                 % define lateral
                    %                 s = p.successor(:);
                    %                 s2 = zeros(ceil(length(s)/6)*6,1);
                    %                 s2(1:length(s)) = s;
                    %                 s2=reshape(s2,ceil(length(s)/6),6);
                    %                 lateral = letter(304,length(s)/2,s2);
                    %                 % define branching zone
                    %                 l = lb + [0; cumsum(ln)];
                    %                 times = zeros(ceil(length(l)/6)*6,1);
                    %                 times2 = zeros(ceil(length(l)/6)*6,1);
                    %                 for j = 1 : length(l)
                    %                     times(j) = growthFunction(l(j),pgf,pos,true);
                    %                     times2(j) = growthFunction(l(j)+la,pgf,pos,true);
                    %                 end
                    %                 delays = times2 - times;
                    %                 times = reshape(times,ceil(length(l)/6),6);
                    %                 delays = reshape(delays,ceil(length(l)/6),6);
                    %                 branching = letter(302,[0,nob,type],[pgf,ptr; rlt, dx, 0, ...
                    %                     pos.R(:,1)'; times; delays; lateral; apical]);
                else
                    branching=apical;
                end
                
                if false %lb>0 % basal zone
                    %                     l0 = 0;
                    %                     lend = lb;
                    %                     t0 = 0;
                    %                     tend = growthFunction(lend,pgf,pos,true);
                    %                     basal = letter(400,[0,0,type,iid],[pgf,ptr;t0,tend,l0,lend,rlt,dx;...
                    %                         pos.R(:,1)' b_age p.theta_dicho(1) p.theta_dicho(2); branching],pos); % section growth
                else
                    basal=branching;
                    basal(2:4,:) = [[pos.x'; pos.C; pos.a, pos.t, 0],pos.R]; % set position
                end
                
                nl = [initial; applyMycRules(basal,dt,s); letter(']')];  %apply time overhead
                
            else
                nl =[];
            end
        else
            nl = [];
        end
        
    otherwise % do nothing
        
        nl = l;
        
end



function pos = getPosition(l)
% getPosition: copies paramters into a struct for better readability
%
pos = struct('x',l(1,1:3)','C',l(2,1:3),'a',l(3,1),'t',l(3,2),'R',l(1:3,4:6));



function pos = updatePos2(dx,t,pos)
% fast position update for single segment
%
R = pos(:,4:6);
a = t(4); % beta
R = R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
a = t(3); % alpha
R = R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
pos(:,4:6) = R;
x = pos(1,1:3);
x = x + dx*R(:,1)';
pos(1,1:3)=x;
