function [A, nodes, indS] = getGraph2(str, pos)
% getGraph: returns a graph representation of the root system
%
% Parameters:
% str           list of l-system letters
% (pos)         initial position (default: pos.x=[0;0;0], pos.R=eye(3))
%
% A             adjacency matrix, entries are edges numbers
% nodes         Nodes are the base of the root, root tips, and branching points.   
%               Branching points are the basess of laterals, or whre
%               laterals potentially appear.                
% indS          list of multiple segment indices corresponding to the edges
%
% See also: fitFitter, getGraph
%
% Copyright 2012-2014 Daniel Leitner. See license.txt for details.
%

if (nargin<2)
    pos.x=[0;0;0];
    pos.R = diag([1 1 1]);
end

sC = 1; % segment index (within edge)
nC = 1; % node index
eC = 1; % edge index
c = 1; % segment counter
stkPtr = 1; % stack pointer

indices(eC,1:2) = [0,0]; % edge list
indS{eC} = []; % segment index list per edge
nodes(nC,:) = pos.x; % nodes

sni = 1; % start node index

i=1;
while i<=size(str,1)
    
    l = str(i,:); % letter i
    
    if l(1)==300 % growing tip = F
        l(1) = 1;
        l(3) = 1e-9; % TODO, i dont know that length
    end
    
    switch l(1)
        case 4 % #
        case 3 % C
        case 2 % f
            pos.x = pos.x + l(3)*pos.R(:,1);
        case 1 % F
            
            if l(3)>0 % if this segment has a length
                pos.x = pos.x + l(3)*pos.R(:,1);
                indS{eC}(sC) = c;
                sC = sC + 1;
                c = c + 1; % segment index
            end
            
        case 7 % +
            a = l(3);
            pos.R = pos.R * [ cos(a) -sin(a) 0; sin(a) cos(a) 0; 0 0 1 ];
        case 8 % -
            a = l(3);
            pos.R = pos.R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
        case 9 % &
            a = l(3);
            pos.R = pos.R * [ cos(a) 0  -sin(a) ; 0 1 0; sin(a) 0 cos(a) ];
        case 10 % ^
            a = l(3);
            pos.R = pos.R * [ cos(a) 0 sin(-a) ; 0 1 0; -sin(a) 0 cos(a)];
        case 11 % \
            a = l(3);
            pos.R = pos.R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
        case 12 % /
            a = l(3);
            pos.R = pos.R * [ 1 0 0; 0 cos(a) sin(a) ; 0 -sin(a) cos(a) ];
        case 13 % |
            pos.R = pos.R * [ cos(pi) sin(pi) 0; -sin(pi) cos(pi) 0; 0 0 1];
        case 14 % r
            a = l(4); % beta
            pos.R = pos.R * [ 1 0 0; 0 cos(a) -sin(a) ; 0 sin(a)  cos(a) ] ;
            a = l(3); % alpha
            pos.R = pos.R * [ cos(a) sin(a) 0; -sin(a) cos(a) 0; 0 0 1 ];
        case 5 % [ push the stack
                                  
            if ~branchempty(str(i:end,:))
                if sC>1
                    nC=nC+1;
                    nodes(nC,:) = pos.x; % new node
                    indices(eC,1:2) = [sni, nC]; % new edge
                    eC=eC+1;
                end
                sni = nC; % start node index
            end
            
            stack(stkPtr).xT = pos.x;
            stack(stkPtr).R = pos.R;
            stack(stkPtr).sni = sni;
            stack(stkPtr).sC = sC;
            stkPtr = stkPtr +1 ;             
            sC=1;
            
        case 6 % ] pop the stack
            
            if sC>1  % if there are segments in this edge: create node
                
                nC=nC+1;
                nodes(nC,:) = pos.x;
                indices(eC,1:2) = [sni, nC];
                eC=eC+1;
                
                stkPtr = stkPtr -1;
                pos.x = stack(stkPtr).xT ;
                pos.R = stack(stkPtr).R;
                sni = stack(stkPtr).sni;
                sC = stack(stkPtr).sC;
                
            else % do nothing
                
                stkPtr = stkPtr -1;
                pos.x = stack(stkPtr).xT ;
                pos.R = stack(stkPtr).R;
                sni = stack(stkPtr).sni;
                sC = stack(stkPtr).sC;
                
            end
            
    end
    
    i = i + l(2); % jump to next letter
    
end

A = sparse(indices(:,1),indices(:,2), 1:eC-1, size(nodes,1),size(nodes,1));



function e = branchempty(str)
% checks if subbranch is empty or not
e=1;
stkPtr=1;
i=1;
while i<=size(str,1)
    l = str(i,:); % letter i
    if l(1)==300 % growing tip = F
        l(1) = 1;
    end
    switch l(1)
        case 1 % F
            if l(3)>0 % if this segment has a length
                e=0;
                break;
            end
        case 5 % [ push the stack
            stkPtr = stkPtr+1;
        case 6 % ] pop the stack
            stkPtr = stkPtr-1;
            if stkPtr==1
                break;
            end
    end
    i = i + l(2); % jump to next letter
end